# -*- coding: utf-8 -*-
# python methods for the object module

import objects
import system
import grids
import math
import helpfile

__doc__= """add-in module for module objects

this module isn't meant to be used directly. everything defined here will be
part of the objects module. 
"""

def set_attributes( object_id, *attributes1, **attributes2 ):
	"Set attributes."

	if len(attributes1) > 0:
		for attr in attributes1[0].keys():
			objects.set_attribute( object_id, attr, attributes1[0][attr] )
	elif attributes2:
		for attr in attributes2.keys():
			objects.set_attribute( object_id, attr, attributes2[attr] )

def set_attribute_for_party( attribute, value ):
	"""Set the attribute for the whole party."""
	party_members = system.get_pcs()

	for member in party_members:
		objects.set_attribute( member, attribute, value )

def set_attribute_for_npcs( attribute, value ):
	"""set the attribute for all NPCs in a zone"""
	npcs = system.get_npcs()

	for npc in npcs:
		objects.set_attribute( npc, attribute, value )

def get_inventory_list( object_id ):
	"""Get list of items in inventory.
	
Items are listed with their type, e.g. "SET_UZI" """

	_list = []
	inventory = objects.get_inventory( object_id )
	for slot in inventory:
		_list = _list + slot[0]
	return _list

def get_inventory_id_list( object_id ):
	"""Get list of items in inventory.
	
Items are listed with their id."""

	_list = []
	inventory = objects.get_inventory_ids( object_id )
	for slot in inventory:
		_list = _list + slot[0]
	return _list

def get_position( object_id ):
	"""Returns the position [x,y,z] of object as list."""
	if (	objects.has_attribute(object_id,"x") and 
		objects.has_attribute(object_id,"y") and
		objects.has_attribute(object_id,"relative_z")):
		
		return [objects.get_attribute(object_id,"x"),objects.get_attribute(object_id,"y"),objects.get_attribute(object_id,"relative_z")]
	else:
		print "object has no attribute 'x', 'y', 'z' or object doesnt exist"
	return None


def add_closed_lock( object_id, lockname, closure_type = "normal" ):
	"""Defines the given lock as locked.
	
object_id - Id of building.
lockname - Name of a lock. 
closure_type - Determines how the lock is protected. There are three protection types:
	"normal" = lock is just locked
	"booby-trap" = lock is protected with explosives
	"lock-trap" = lock is protected with razor blades
	"poison-trap" = lock poisons the thief"""

	locked = []
	# check for "locked" attribute of the building
	# this attribute contains all closed locks
	if not objects.has_attribute( object_id, "locked" ):
		locked = []
	else:
		locked = objects.get_attribute( object_id, "locked" )

	dummylist = []

	# get list of dummies that represent locks
	found_lock = False
	object_type = objects.get_attribute( object_id, "typeid" )
	if objects.has_attribute( object_type, "lock_data" ):
		lock_data = objects.get_attribute( object_type, "lock_data" )
		
		# get the dummies of the given lockname
		if lock_data.has_key( lockname ):
			found_lock = True
			dummylist = lock_data[ lockname ][ "dummies" ]

	if not found_lock:
		print "No lock with the name ", lockname ," found."
		print "Lock names look like: lock1, lock2 etc."
		print "If your lockname is like the examples above then really no lock"
		print "with that name exists."
			
	# if the lock has dummies than add them to the locked list.
	if len( dummylist ) > 0:
		for dummy in dummylist:
			locked.append( dummy )

		# make the connections for the door unusable
		if objects.has_attribute( object_type, "lock_connections" ):
			lock_connections = objects.get_attribute( object_type, "lock_connections" )
			if lock_connections.has_key( lockname ):
				for connection in lock_connections[ lockname ]:
					grids.set_connection_usable( connection[0], connection[1], False )

		objects.set_attribute( object_id, "locked", locked )

		lock_data[ lockname ][ "closure_type" ] = closure_type

		objects.set_attribute( object_id, "lock_data", lock_data )
	else:
		print "The given lock exists in the lock list, but has no dummies assigned."
		print "The context menu for cracking a lock will not be shown without any dummies"
		print "assigned to this lock"

	# pass the locked attribute to every interior of the object
	if objects.has_attribute( object_id, "attached_interior_objects" ):
		interior_objects = objects.get_attribute( object_id, "attached_interior_objects" )

		for interior in interior_objects:
			objects.set_attribute( interior, "locked", locked )

def get_lock_name_from_dummy( object_id, dummy ):
	"""Gets the name of the lock from the given dummy.

object_id - Id of the object.
dummy - Name of the dummy."""

	lock = ""

	if objects.has_attribute( object_id, "exterior_object_id" ):
		object_id = objects.get_attribute( object_id, "exterior_object_id" )
		
	object_type = objects.get_attribute( object_id, "typeid" )
	if objects.has_attribute( object_type, "lock_data" ):
		lock_data = objects.get_attribute( object_type, "lock_data" )

		# search the lock by the given dummy
		for key in lock_data.keys():
			if dummy in lock_data[ key ][ "dummies" ]:
				lock = key
	
	return lock

def has_closed_lock( object_id, lockname = None ):
	"""Checks if the given object has a closed lock of the name lockname.

object_id - Id of the object.
lockname - Name of the lock. If not set then the 
		function just checks if the object
		has any lock closed, no matter which.
"""
	if objects.has_attribute( object_id, "locked" ):
		locked = objects.get_attribute( object_id, "locked" )
		if lockname == None:
			if len( locked ) > 0:
				return True
		else:
			for dummy in locked:
				if get_lock_name_from_dummy(object_id, dummy) == lockname:
					return True
			
	return False

def remove_closed_lock( object_id, dummy ):
	"""Removes the lock that is determined by the given dummy from the locked list,
and also makes the grid connection usable.

object_id - Id of the building in which the dummy is.
dummy - This dummy determines the cracked door that should be removed.
"""
	# check for "locked" attribute of the building
	# this attribute contains all closed locks
	if not objects.has_attribute( object_id, "locked" ):
		print "The object does not contain any closed locks."
		return

	# get list of dummies that represent locks
	# and search for the lockname that is determined by the
	# given dummy.
	lockname = None
	if objects.has_attribute( object_id, "exterior_object_id" ):
		object_id = objects.get_attribute( object_id, "exterior_object_id" )

	object_type = objects.get_attribute( object_id, "typeid" )
	if objects.has_attribute( object_type, "lock_data" ):
		lock_data = objects.get_attribute( object_type, "lock_data" )
		# search the lock by the given dummy
		for key in lock_data.keys():
			if dummy in lock_data[ key ][ "dummies" ]:
				lockname = key
				break

	if lockname == None:
		print "No lock found that is assigned to the given dummy."
		return
	else:
		remove_closed_lock_by_name( object_id, lockname )

def remove_closed_lock_by_name( object_id, lock_name ):
	"""Removes the lock that is determined by the given lock_name
from the locked list, and also makes the grid connection
usable.

object_id - Id of the building in which the dummy is.
lock_name - This name determines the cracked lock that should be removed.
"""

	lockname = lock_name 

	# check for "locked" attribute of the building
	# this attribute contains all closed locks
	if not objects.has_attribute( object_id, "locked" ):
		print "The object does not contain any closed locks."
		return

	if objects.has_attribute( object_id, "exterior_object_id" ):
		object_id = objects.get_attribute( object_id, "exterior_object_id" )

	lock_data = {}
	locked = objects.get_attribute( object_id, "locked" )
	object_type = objects.get_attribute( object_id, "typeid" )
	if objects.has_attribute( object_type, "lock_data" ):
		lock_data = objects.get_attribute( object_type, "lock_data" )

	if lockname != None:
		# remove all dummies of the lock that is determined
		# by the given dummy from the locked list.
		for lock_dummy in lock_data[ lockname ][ "dummies" ]:
			if lock_dummy in locked:
				locked.remove( lock_dummy )

		# make the connections for the door usable
		if objects.has_attribute( object_type, "lock_connections" ):
			lock_connections = objects.get_attribute( object_type, "lock_connections" )
			if lock_connections.has_key( lockname ):
				for connection in lock_connections[ lockname ]:
					grids.set_connection_usable( connection[0], connection[1], True )

		# renew the locked list
		objects.set_attribute( object_id, "locked", locked )

		# pass the locked attribute to every interior of the object
		if objects.has_attribute( object_id, "attached_interior_objects" ):
			interior_objects = objects.get_attribute( object_id, "attached_interior_objects" )

			for interior in interior_objects:
				objects.set_attribute( interior, "locked", locked )
	else:
		print "No lock found that is assigned to the given dummy."
				
def restore_locks( object_id ):
	"""Sets all locked connections to unusable so it matches the object's 
attributes.
	
object_id - Id of building.
"""

	locked = []
	# check for "locked" attribute of the building
	# this attribute contains all closed locks
	if not objects.has_attribute( object_id, "locked" ):
		locked = []
	else:
		locked = objects.get_attribute( object_id, "locked" )
	
	object_type = objects.get_attribute( object_id, "typeid" )
		
	for dummy in locked:
		lock_name = get_lock_name_from_dummy( object_id, dummy )
		
		# make the connections for the door unusable
		if objects.has_attribute( object_type, "lock_connections" ):
			lock_connections = objects.get_attribute( object_type, "lock_connections" )
			if lock_connections.has_key( lock_name ):
				for connection in lock_connections[ lock_name ]:
					grids.set_connection_usable( connection[0], connection[1], False )

def get_direction_to_object(char, obj):
	"""Returns the direction from char to obj in degrees.
	
The return value is such that the char is facing the obj if:
objects.turn_to_direction(char, objects.get_direction_to_object(char, obj)).
"""
	return get_direction_to_position(char, objects.get_x(obj), objects.get_y(obj))

def get_direction_to_position(char, x, y):
	"""Returns the direction from char to obj in degrees.
	
The return value is such that the char is facing the position if:
objects.turn_to_direction(char, objects.get_direction_to_position(char, x, y))
"""
	char_x = objects.get_x(char)
	char_y = objects.get_y(char)
	target_x = x
	target_y = y
	xdiff = target_x - char_x
	ydiff = target_y - char_y
	radius = math.sqrt(xdiff**2 + ydiff**2)
	if radius==0: return 0 # both chars are at the same position
	if xdiff>=0:
		angle = math.pi + math.acos(ydiff/radius)
	else:
		angle = math.pi - math.acos(ydiff/radius)
	angle *= -1
	while angle < 0: angle += 2*math.pi
	while angle > math.pi*2: angle -= 2*math.pi
	return math.degrees(angle)

def get_distance_to_object(char, obj):
	"""Returns the distance from char to obj."""
	return get_distance_to_position(char, objects.get_x(obj), objects.get_y(obj))

def get_distance_to_position(obj, x, y):
	"""Returns the distance from obj to x,y."""
	object_x = objects.get_x(obj)
	object_y = objects.get_y(obj)
	target_x = x
	target_y = y
	xdiff = target_x - object_x
	ydiff = target_y - object_y
	return math.sqrt(xdiff**2 + ydiff**2)

def can_do_npc_ki_action(char):
	"""Returns True if the char is an NPC and can do a ki action."""
	if char not in system.get_npcs(): return False
	if not objects.is_active(char): return False
	if objects.is_destroyed(char): return False
	return True

def put_ammo_into_inventory(char, ammo_type, bullet_count):
	"""Puts ammo into the inventory. Non-empty ammo packs of same type are filled first
before new ammo packs are created."""
	l = lambda item: objects.has_attribute(item,"typeid") and objects.get_attribute(item,"typeid")==ammo_type
	ammos = filter(l, objects.get_inventory_id_list(char))
	max_pack_count = objects.get_attribute(ammo_type, "schuss")
	for ammo in ammos:
		# try to fill existing ammo packs of same type, if they aren't full
		count_in_pack = objects.get_attribute(ammo, "act_ammo_count")
		empty_space = max_pack_count - count_in_pack
		fill = min(empty_space, bullet_count)
		if fill:
			count_in_pack += fill
			objects.set_attribute(ammo, "act_ammo_count", count_in_pack)
			bullet_count -= fill
	while bullet_count:
		# still ammo to store? create new ammo packs til we have all ammo spend
		ammo = objects.create_item_in_inventory(char, ammo_type)
		count_in_pack = min(max_pack_count, bullet_count)
		bullet_count -= count_in_pack
		if count_in_pack < max_pack_count:
			objects.set_attribute(ammo, "act_ammo_count", count_in_pack)

def is_alive_and_active(char):
	"""Returns True if the char is alive and active and can do an action.

Parameters:
        char - ID of char.
"""
	
	if not char in system.get_npcs() + system.get_pcs() + system.get_animals():
		raise ValueError, " PC, NPC or ANIMAL: '%s' is not in the current zone" %char
		
	if objects.is_destroyed(char):
		return False
	if not objects.is_active(char):
		return False
	if objects.is_unconscious(char):
		return False
		
	return True
	
def activate_mine(mine, immune=[], bomb_owner=None):
	"""Activates a mine on the ground.

Nothing happens if the mine is still in an inventory.

Parameters:
        mine - Id of the mine
        immune - List with party ids or object ids that don't trigger the mine.
                 e.g. ["UISPIELER"] to exclude the player party
                 can be [] to hit everyone
        bomb_owner - who has set the mine, can be None
"""
	if not objects.is_in_inventory(mine):
		objects.set_attribute(mine, "mine_active", True)
		objects.set_attribute(mine, "immune", immune)
		objects.set_attribute(mine, "bomb_owner", bomb_owner)
		objects.set_attribute(mine, "bomb_is_sharp", True)
	
def deactivate_mine(mine):
	"""Deactivates a mine."""
	objects.set_attribute(mine, "mine_active", False)
	objects.set_attribute(mine, "immune", None)
	objects.set_attribute(mine, "bomb_owner", None)
	objects.set_attribute(mine, "bomb_is_sharp", False)

#derff's hack's is here. Start

def get_att(obj, att):
	if not objects.has_attribute(obj, att): return None
	return objects.get_attribute(obj, att)

def backpack_check(backpack):
	print "backpack_check>", objects.get_att(backpack,"itemer") 
	if objects.get_att(backpack,"itemer") != None:
		objects.set_attribute(backpack,"itemer",None)
		itemer = helpfile.create_itemer(backpack)
		inv = objects.get_inventory_id_list (itemer)
		for item in inv:
			objects.transfer_item(itemer, item, backpack)

def update_backpack(char,backpack):
	"""Update parameters of backpack and character after transfer completion	
"""
	import object_events
	object_events.update_backpack(char,backpack)

def Made_Time_From_Int(float):
	if float == None : return None
	msec = float % 1000 
	other = ( float - msec ) / 1000 
	sec = other % 60
	other = ( other - sec ) / 60
	min = other % 60
	hou = ( other - min ) / 60
	Sting = "" + str(hou) + ":" + str(min) + ":" + str(sec)
	return Sting

def Calculate_Weight_With_Charges ( item_id ):
	"""Function return weight of item WITH charges, so water cans weight become adequate
"""
	# print "weight>", item_id, objects.get_att (item_id, "weight") 
 	base_weight = objects.get_att ( item_id, "weight" )
	if ( base_weight == None ) or \
		not ( objects.has_attribute ( item_id, "charge" ) or objects.has_attribute ( item_id, "container" ) \
			or objects.has_attribute ( item_id, "fuel" ) or objects.has_attribute ( item_id, "weapon_type" ) ) :
		return base_weight 

	charge_number = 0
	charge_weight = 0
	if objects.has_attribute ( item_id, "charge" ) : #water and healing
		# print "charge"
		charge_number = objects.get_attribute ( item_id, "charge" ) 
		if objects.has_attribute ( item_id, "water_repository" ) :
			charge_weight = 0.0 ; # weight for water is 1.0 for litter	# [BrainRipper]
		elif objects.has_attribute ( item_id, "le_recovery" ) :
			charge_weight = 0.1 ; # weight for healing is 0.1 
		else: charge_weight = 0.0
	elif objects.has_attribute ( item_id, "fuel" ) : 
		# print "fuel"	
		charge_number = objects.get_attribute ( item_id, "fuel" ) 
		charge_weight = 0.75 ; # weight for fuel is 0.75 kg for litter
	elif objects.has_attribute ( item_id, "container" ) :
		# print "container"
		type_id = objects.get_attribute ( item_id, "typeid" )
		if objects.get_att ( type_id, "weight" ) != base_weight : # Hack for previous version of backpack
			base_weight = objects.get_att ( type_id, "weight" )
			objects.set_attribute ( item_id, "weight", base_weight )
			if base_weight == None : return base_weight

			inventory = objects.get_attribute ( item_id, "container" )
			inside_weight = 0
			for item in inventory :
				if item[0] != "" and objects.has_attribute( item[0], "weight"):
					inside_weight += objects.Calculate_Weight_With_Charges ( item[0] ) * item[1]
			objects.set_attribute ( item_id, "inside_weight", inside_weight )
			
		charge_weight = objects.get_att ( item_id, "inside_weight" )
		if ( charge_weight == None ): charge_weight = 0
		charge_number = objects.get_att( item_id, "backpack_weight_bonus" )
		if charge_number == None: charge_number  = 1

	elif  objects.has_attribute ( item_id, "weapon_type" ): # weapon
		# print "ammo"
		charge_number = objects.get_attribute ( item_id, "act_ammo_count" ) 
		ammo_type = objects.get_att ( item_id, "act_ammo_type" )
		if ammo_type == None or ammo_type == "" or ammo_type.endswith("_PSEUDOAMMO") : return base_weight # ammo type SHOULD be defined
		ammo_weight = objects.get_att ( ammo_type, "weight" )
		if ammo_weight == None: ammo_weight = 0 
		ammo_count = objects.get_att ( ammo_type, "schuss" )
		if ammo_count == None or ammo_count < 1 : return base_weight 
		charge_weight = ammo_weight / ammo_count

		if objects.has_attribute ( item_id, "saved_act_ammo_count" ) :
			charge_number_2 = 0
			charge_weight_2 = 0
			ammo_type_2 = objects.get_att ( item_id, "saved_act_ammo_type" )
			if not ( ammo_type_2 == None or ammo_type_2 == "" ) : 
				charge_number_2 = objects.get_att ( item_id, "saved_act_ammo_count" ) 
				ammo_weight_2 = objects.get_att ( ammo_type_2, "weight" )
				if ammo_weight_2 == None: ammo_weight_2 = 0 
				ammo_count_2 = objects.get_att ( ammo_type_2, "schuss" )
				if ammo_count_2 == None or ammo_count_2 < 1 : ammo_count_2 = 0
				charge_weight_2 = ammo_weight_2 / ammo_count_2
				base_weight += charge_number_2 * charge_weight_2

	return base_weight + charge_number * charge_weight

#derff's hack's is here. 

import sys

# add methods to object module
sys.modules["objects"].set_attributes = set_attributes
sys.modules["objects"].set_attribute_for_party = set_attribute_for_party
sys.modules["objects"].get_inventory_list = get_inventory_list
sys.modules["objects"].get_inventory_id_list = get_inventory_id_list
sys.modules["objects"].get_position = get_position
sys.modules["objects"].add_closed_lock = add_closed_lock
sys.modules["objects"].has_closed_lock = has_closed_lock
sys.modules["objects"].remove_closed_lock = remove_closed_lock
sys.modules["objects"].remove_closed_lock_by_name = remove_closed_lock_by_name
sys.modules["objects"].restore_locks = restore_locks
sys.modules["objects"].get_lock_name_from_dummy = get_lock_name_from_dummy
sys.modules["objects"].get_direction_to_object = get_direction_to_object
sys.modules["objects"].get_direction_to_position = get_direction_to_position
sys.modules["objects"].get_distance_to_position = get_distance_to_position
sys.modules["objects"].get_distance_to_object = get_distance_to_object
sys.modules["objects"].get_distance = get_distance_to_object
sys.modules["objects"].can_do_npc_ki_action = can_do_npc_ki_action
sys.modules["objects"].put_ammo_into_inventory = put_ammo_into_inventory
sys.modules["objects"].is_alive_and_active = is_alive_and_active
sys.modules["objects"].activate_mine = activate_mine
sys.modules["objects"].deactivate_mine = deactivate_mine
sys.modules["objects"].get_att = get_att
sys.modules["objects"].backpack_check = backpack_check
sys.modules["objects"].update_backpack = update_backpack
sys.modules["objects"].Made_Time_From_Int = Made_Time_From_Int
sys.modules["objects"].Calculate_Weight_With_Charges = Calculate_Weight_With_Charges
